
--Configuration
local barTexture = "Interface\\AddOns\\MicroHUD\\Statusbar_Clean"
local barFont = "GameFontNormal"
local smallBarFont = "GameFontSmall"
local buffFont = "GameFontWhiteTiny"

local barWidth = 200
local barHeight = 20

local vSpace = 1
local hOffset = barWidth * 1.3


--Colorize the standard nameplates based on threat.
local isTank = function(t)
    return UnitGroupRolesAssigned(t) == "TANK"
end


local isOfftanked = function(frame)
	local t2 = frame.displayedUnit.."target"

    if UnitPlayerOrPetInRaid(t2) or UnitPlayerOrPetInParty(t2) then
        if not UnitIsUnit("player", t2) and isTank(t2) and isTank("player") then
            return true
        end
    end

    return false
end


local threatColor = function(s)
	if isTank("player") then
		if s == 0 then
			return 1.0, 0.0, 0.0
		elseif s == 1 then
			return 1.0, 0.5, 0.0
		elseif s == 2 then
			return 1.0, 1.0, 0.0
		else
			return 0.0, 1.0, 0.0
		end
	else
		if s == 0 then
			return 0.0, 1.0, 0.0
		elseif s == 1 then
			return 1.0, 1.0, 0.0
		elseif s == 2 then
			return 1.0, 0.5, 0.0
		else
			return 1.0, 0.0, 0.0
		end
	end
end


hooksecurefunc("CompactUnitFrame_UpdateHealthColor", function(frame)
	local r, g, b

	if frame == nil then
		return
	elseif frame.displayedUnit == nil then
		return
	elseif C_NamePlate.GetNamePlateForUnit(frame.unit) == C_NamePlate.GetNamePlateForUnit("player") then
		return
	elseif not UnitIsConnected(frame.displayedUnit) then
		r, g, b = 0.5, 0.5, 0.5
	else
		local col = RAID_CLASS_COLORS[select(2, UnitClass(frame.displayedUnit))]

		if CompactUnitFrame_IsTapDenied(frame) then
			r, g, b = 0.1, 0.1, 0.1
		elseif UnitIsPlayer(frame.displayedUnit) and col then
			r, g, b = col.r, col.g, col.b
		elseif CompactUnitFrame_IsOnThreatListWithPlayer(frame.displayedUnit) then
			local s = UnitThreatSituation("player", frame.displayedUnit)

			if isOfftanked(frame) then
				if s > 0 then
					r, g, b = 1.0, 0.5, 1.0
				else
					r, g, b = 0.0, 0.5, 1.0
				end
			else
				r, g, b = threatColor(s)
			end
		elseif isOfftanked(frame) then
			r, g, b = 0.0, 0.5, 1.0
		else
			r, g, b = UnitSelectionColor(frame.displayedUnit)
		end
	end

	frame.healthBar:SetStatusBarColor(r, g, b)
end)


--Get class color of unit.
local function cc(u)
	local col = {r = 1, g = 0, b = 0}
	local _, c = UnitClass(u)

	if c and UnitIsPlayer(u) then
		col = RAID_CLASS_COLORS[c]
	end

	return col
end


--Format large health and power numbers.
local function sn(h)
	if abs(h) > 999999999 then
		return format("%.2fb", h / 1000000000.0)
	elseif abs(h) > 999999 then
		return format("%.2fm", h / 1000000.0)
	elseif abs(h) > 999 then
		return format("%.2fk", h / 1000.0)
	else
		return format("%d", h)
	end
end


--Format bar text from Current and Max Values.
local function bt(c, m)
	if m > 0 and c >= 0 and (not (m == 100.0)) then
		return format("%s/%s  %.2f%%", sn(c), sn(m), (100.0 * c) / m)
	end

	return format("%s/%s", sn(c), sn(m))
end


--Left side, player health.
local ph = CreateFrame("StatusBar")
ph:SetWidth(barWidth)
ph:SetHeight(2 * barHeight)
ph:SetPoint("CENTER", UIParent, "CENTER", -hOffset, 0)
ph:SetStatusBarTexture(barTexture)
ph:SetStatusBarColor(0, 0.75, 0)
ph:RegisterEvent("PLAYER_ENTERING_WORLD")
ph:RegisterEvent("UNIT_HEALTH")
ph:RegisterEvent("UNIT_MAXHEALTH")

local phb = ph:CreateTexture(nil, "BACKGROUND")
phb:SetAllPoints()
phb:SetColorTexture(0, 0, 0, 0.5)

local pht = ph:CreateFontString()
pht:SetFontObject(barFont)
pht:SetPoint("CENTER")

ph:SetScript("OnEvent",
	function(f, e, ...)
		f:SetMinMaxValues(0, UnitHealthMax("player"))
		f:SetValue(UnitHealth("player"))
		pht:SetText(UnitName("player") .. "\n" .. bt(UnitHealth("player"), UnitHealthMax("player")))
	end
)


--Left side, player power.
local pp = CreateFrame("StatusBar")
pp:SetWidth(barWidth)
pp:SetHeight(barHeight)
pp:SetPoint("TOP", ph, "BOTTOM", 0, -vSpace)
pp:SetStatusBarTexture(barTexture)
pp:RegisterEvent("PLAYER_ENTERING_WORLD")
pp:RegisterEvent("UNIT_DISPLAYPOWER")
pp:RegisterEvent("UNIT_POWER_UPDATE")
pp:RegisterEvent("UNIT_MAXPOWER")

local ppb = pp:CreateTexture(nil, "BACKGROUND")
ppb:SetAllPoints()
ppb:SetColorTexture(0, 0, 0, 0.5)

local ppt = pp:CreateFontString()
ppt:SetFontObject(barFont)
ppt:SetPoint("CENTER")

pp:SetScript("OnEvent",
	function(f, e, ...)
		local col = PowerBarColor[UnitPowerType("player")]
		  
		f:SetStatusBarColor(col.r, col.g, col.b)
		f:SetMinMaxValues(0, UnitPowerMax("player"))
		f:SetValue(UnitPower("player"))
		ppt:SetText(bt(UnitPower("player"), UnitPowerMax("player")))
	end
)


--Left side, player absorbs.
local ps = CreateFrame("StatusBar")
ps:SetWidth(barWidth)
ps:SetHeight(barHeight)
ps:SetPoint("TOP", pp, "BOTTOM", 0, -vSpace)
ps:SetStatusBarTexture(barTexture)
ps:SetStatusBarColor(0.8, 0.8, 0.8)
ps:RegisterEvent("PLAYER_ENTERING_WORLD")
ps:RegisterEvent("UNIT_ABSORB_AMOUNT_CHANGED")
ps:RegisterEvent("UNIT_MAXHEALTH")

local psb = ps:CreateTexture(nil, "BACKGROUND")
psb:SetAllPoints()
psb:SetColorTexture(0, 0, 0, 0.5)

local pst = ps:CreateFontString()
pst:SetFontObject(barFont)
pst:SetPoint("CENTER")

ps:SetScript("OnEvent",
	function(f, e, ...)
		local m = UnitHealthMax("player")
		local a = UnitGetTotalAbsorbs("player")

		f:SetMinMaxValues(0, m)
		f:SetValue(a)
		pst:SetText(bt(a, m))

		if a > 0 then
			if a > m then
				f:SetStatusBarColor(1, 1, 1)
			else
				f:SetStatusBarColor(0.8, 0.8, 0.8333333)
			end

			f:Show()
		else
			f:Hide()
		end
	end
)


--Right side, target health.
local th = CreateFrame("StatusBar")
th:SetWidth(barWidth)
th:SetHeight(2 * barHeight)
th:SetPoint("CENTER", UIParent, "CENTER", hOffset, 0)
th:SetStatusBarTexture(barTexture)
th:SetStatusBarColor(0.75, 0, 0)
th:RegisterEvent("PLAYER_ENTERING_WORLD")
th:RegisterEvent("PLAYER_TARGET_CHANGED")
th:RegisterEvent("RAID_TARGET_UPDATE")
th:RegisterEvent("UNIT_NAME_UPDATE")
th:RegisterEvent("UNIT_HEALTH")
th:RegisterEvent("UNIT_MAXHEALTH")
th:Hide()

local thb = th:CreateTexture(nil, "BACKGROUND")
thb:SetAllPoints()
thb:SetColorTexture(0, 0, 0, 0.5)

local tht = th:CreateFontString()
tht:SetFontObject(barFont)
tht:SetPoint("CENTER")

local ri = UIParent:CreateTexture()
ri:SetHeight(barHeight * 2)
ri:SetWidth(barHeight * 2)
ri:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
ri:SetPoint("RIGHT", th, "LEFT", -2 * vSpace, 0)
ri:Hide()

th:SetScript("OnEvent",
	function(f, e, ...)
		if UnitExists("target") and UnitHealthMax("target") > 0 then
			local i = GetRaidTargetIndex("target")
			local c = cc("target")

			if i then
			  SetRaidTargetIconTexture(ri, i)
			  ri:Show()
			else
			  ri:Hide()
			end

			f:SetStatusBarColor(c.r, c.g, c.b)
			f:SetMinMaxValues(0, UnitHealthMax("target"))
			f:SetValue(UnitHealth("target"))
			tht:SetText(UnitName("target") .. "\n" .. bt(UnitHealth("target"), UnitHealthMax("target")))
			f:Show()
		else
			f:Hide()
			ri:Hide()
		end
	end
)


--Right side, target power.
local tp = CreateFrame("StatusBar")
tp:SetWidth(barWidth)
tp:SetHeight(barHeight)
tp:SetPoint("TOP", th, "BOTTOM", 0, -vSpace)
tp:SetStatusBarTexture(barTexture)
tp:RegisterEvent("PLAYER_ENTERING_WORLD")
tp:RegisterEvent("PLAYER_TARGET_CHANGED")
tp:RegisterEvent("UNIT_DISPLAYPOWER")
tp:RegisterEvent("UNIT_POWER_UPDATE")
tp:RegisterEvent("UNIT_MAXPOWER")
tp:Hide()

local tpb = tp:CreateTexture(nil, "BACKGROUND")
tpb:SetAllPoints()
tpb:SetColorTexture(0, 0, 0, 0.5)

local tpt = tp:CreateFontString()
tpt:SetFontObject(barFont)
tpt:SetPoint("CENTER")

tp:SetScript("OnEvent",
	function(f, e, ...)
		if UnitExists("target") and UnitPowerMax("target") > 0 then
			local col = PowerBarColor[UnitPowerType("target")]

			f:SetStatusBarColor(col.r, col.g, col.b)
			f:SetMinMaxValues(0, UnitPowerMax("target"))
			f:SetValue(UnitPower("target"))
			tpt:SetText(bt(UnitPower("target"), UnitPowerMax("target")))
			f:Show()
		else
			f:Hide()
		end
	end
)


--Right side threat information.
local tt = CreateFrame("StatusBar")
tt:SetWidth(barWidth)
tt:SetHeight(barHeight)
tt:SetPoint("TOP", tp, "BOTTOM", 0, -vSpace)
tt:SetStatusBarTexture(barTexture)
tt:SetMinMaxValues(0, 100)
tt:RegisterEvent("PLAYER_ENTERING_WORLD")
tt:RegisterEvent("PLAYER_TARGET_CHANGED")
tt:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
tt:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
tt:Hide()

local ttb = tt:CreateTexture(nil, "BACKGROUND")
ttb:SetAllPoints()
ttb:SetColorTexture(0, 0, 0, 0.5)

local ttt = tt:CreateFontString()
ttt:SetFontObject(barFont)
ttt:SetPoint("CENTER")

tt:SetScript("OnEvent",
	function(f, e, ...)
		if UnitExists("target") then
			local _, s, p, _, _ = UnitDetailedThreatSituation("player", "target")
			local r, g, b = GetThreatStatusColor(s)
			local sdisplay = ""

			if s == nil or p == nil then
				f:Hide()
			else
				f:SetStatusBarColor(r, g, b)
				f:SetValue(p)

				if s == 0 then
					ttt:SetText("")
				elseif s == 1 then
					ttt:SetText("TAKING AGGRO")
				elseif s == 2 then
					ttt:SetText("LOSING AGGRO")
				elseif s == 3 then
					ttt:SetText("TANKING")
				end

				f:Show()
			end
		else
			f:Hide()
		end
	end
)


--Center target casting.
local cf = CreateFrame("StatusBar")
cf:SetWidth(barWidth)
cf:SetHeight(barHeight)
cf:SetPoint("CENTER", UIParent, "CENTER", 0, 2 * barHeight)
cf:SetStatusBarTexture(barTexture)
cf:SetStatusBarColor(0, 0, 0, 0)

local cfb = cf:CreateTexture(nil, "BACKGROUND")
cfb:SetAllPoints()

local cft = cf:CreateFontString()
cft:SetFontObject(barFont)
cft:SetTextColor(0, 0, 0, 0)
cft:SetPoint("CENTER")

local refreshCasting = function()
	local spell, displayName, _, startTime, endTime, _, _, interrupt = UnitCastingInfo("target")

	if not spell then
		spell, displayName, _, startTime, endTime, _, interrupt = UnitChannelInfo("target")
	end

	if spell then
		cft:SetText(displayName)
		cf:SetMinMaxValues(startTime, endTime)
		cf:SetValue(1000 * GetTime())

		if not interrupt then
			cf:SetStatusBarColor(0, 1, 0, 1)
			cft:SetTextColor(1, 1, 1, 1)
			cfb:SetColorTexture(0, 0, 0, 1)
		else
			cf:SetStatusBarColor(1, 0, 0, 1)
			cft:SetTextColor(1, 1, 1, 1)
			cfb:SetColorTexture(0, 0, 0, 1)
		end
	else
		cf:SetStatusBarColor(0, 0, 0, 0)
		cft:SetTextColor(0, 0, 0, 0)
		cfb:SetColorTexture(0, 0, 0, 0)
	end
end

local castingTimer = C_Timer.NewTicker(0.1, refreshCasting, nil)
